from django import forms
from django.contrib import admin
from django.contrib.admin.forms import AdminAuthenticationForm
from django.contrib.auth import authenticate
from django.contrib.auth.admin import UserAdmin

from .models import CustomUser


class OTPAdminAuthenticationForm(AdminAuthenticationForm):
    otp = forms.CharField(required=False)

    def clean(self):
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')
        otp = self.cleaned_data.get('otp')

        if username is not None and password:
            self.user_cache = authenticate(self.request, username=username, password=password, otp=otp)
            if self.user_cache is None:
                raise self.get_invalid_login_error()
            else:
                self.confirm_login_allowed(self.user_cache)

        return self.cleaned_data


class CustomUserAdmin(UserAdmin):
    model = CustomUser

    fieldsets = (

        ('Personal info', {
            'fields': (
                'first_name',
                'last_name',
                'userpic',
                'preferred_language',)
        }),
        ('Contacts', {'fields': (
            'email',
            'email_candidate',
            'is_email_confirmed',
            'email_confirm_pin',
            'email_candidate_confirm_pin',
            'email_confirm_attempts',
            'phone_number',
            'phone_number_candidate',
            'is_phone_number_confirmed',
            'phone_number_confirm_pin',
            'phone_number_candidate_confirm_pin',
            'phone_number_confirm_attempts',
            'last_password_reset_sms_sent_at',
        )}),
        ('Access', {
            'fields': (
                'is_active',
                'is_staff'
            ),
        }),
        ('Security', {'fields': ('password', 'mfa_secret_key')}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('phone_number', 'email', 'password1', 'password2'),
        }),
    )

    list_display = (
        'id', 'first_name', 'last_name', 'phone_number', 'email', 'created_at')

    ordering = ('-created_at',)
    list_filter = ('is_staff', 'is_superuser', 'is_active', 'groups')
    search_fields = ('phone_number', 'first_name', 'last_name', 'email', 'id')


admin.site.register(CustomUser, CustomUserAdmin)



import random
import uuid
from datetime import timedelta
from enum import Enum

import pyotp as pyotp
from django.conf import settings as django_settings
from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
from django.contrib.auth.models import PermissionsMixin
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone, translation
from django.utils.translation import gettext as _

from moses.conf import settings as moses_settings


class CustomUserManager(BaseUserManager):
    use_in_migrations = True

    def _create_user(self, phone_number, password, **extra_fields):
        user = self.model(phone_number=phone_number, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, phone_number=None, password=None, **extra_fields):
        extra_fields.setdefault('is_staff', False)
        extra_fields.setdefault('is_superuser', False)
        extra_fields.setdefault('is_active', True)
        user = self._create_user(phone_number, password, **extra_fields)
        user.send_phone_number_confirmation_sms(generate_new=True)
        user.send_email_confirmation_email(generate_new=True)
        return user

    def create_superuser(self, phone_number, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')

        return self._create_user(phone_number, password, **extra_fields)


class Credential(Enum):
    EMAIL = 'email'
    PHONE_NUMBER = 'phone_number'


class CustomUser(AbstractBaseUser, PermissionsMixin):
    class Meta:
        verbose_name = _("User")
        verbose_name_plural = _("Users")
        constraints = [
            models.UniqueConstraint(fields=['site', 'phone_number'], name='one_phone_number_per_site'),
            models.UniqueConstraint(fields=['site', 'email'], name='one_email_per_site')
        ]

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    site = models.ForeignKey(
        'sites.Site',
        related_name='users',
        on_delete=models.CASCADE
    )

    email = models.EmailField(blank=False)
    email_candidate = models.EmailField(blank=True, verbose_name=_("Email candidate"))
    is_email_confirmed = models.BooleanField(default=False, verbose_name=_("Is email confirmed"))
    email_confirmation_pin = models.PositiveIntegerField(default=0, verbose_name=_("Email confirm PIN"))
    email_candidate_confirmation_pin = models.PositiveIntegerField(default=0,
                                                                   verbose_name=_("Email candidate confirm PIN"))
    email_confirmation_attempts = models.PositiveSmallIntegerField(default=0, verbose_name=_("Email confirm attempts"))

    phone_number = models.CharField(max_length=20, verbose_name=_("Phone number"))
    phone_number_candidate = models.CharField(max_length=20, blank=True, verbose_name=_("Phone number candidate"))
    is_phone_number_confirmed = models.BooleanField(default=False, verbose_name=_("Is phone number confirmed"))
    phone_number_confirmation_pin = models.PositiveIntegerField(default=0, verbose_name=_("Phone number confirm PIN"))
    phone_number_candidate_confirmation_pin = models.PositiveIntegerField(default=0, verbose_name=_(
        "Phone number candidate confirm PIN"))
    phone_number_confirmation_attempts = models.PositiveSmallIntegerField(
        default=0,
        verbose_name=_("Phone number confirm attempts")
    )
    last_password_reset_sms_sent_at = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_("Last password reset sms sent at")
    )

    first_name = models.CharField(max_length=200, verbose_name=_("First name"), blank=True)
    last_name = models.CharField(max_length=200, verbose_name=_("Last name"), blank=True)

    is_active = models.BooleanField(default=True, verbose_name=_("Is active"))
    is_staff = models.BooleanField(default=False, verbose_name=_("Is staff"))

    preferred_language = models.CharField(
        choices=moses_settings.LANGUAGE_CHOICES,
        default=moses_settings.DEFAULT_LANGUAGE,
        max_length=10,
        verbose_name=_("Preferred language")
    )
    created_at = models.DateTimeField(default=timezone.now, blank=True, null=True, verbose_name=_("Created at"))

    mfa_secret_key = models.CharField(blank=True, default='', max_length=160)
    last_phone_number_confirmation_pin_sent = models.DateTimeField(null=True, blank=True)
    last_phone_number_candidate_confirmation_pin_sent = models.DateTimeField(null=True, blank=True)
    userpic = models.ImageField(upload_to='images/userpics/', blank=True, null=True)

    USERNAME_FIELD = 'phone_number'

    objects = CustomUserManager()

    def check_mfa_otp(self, otp):
        if self.is_superuser and not django_settings.DEBUG and not self.mfa_secret_key:
            return False
        elif not self.mfa_secret_key:
            return True
        totp = pyotp.totp.TOTP(self.mfa_secret_key.encode('utf-8'))
        return totp.verify(otp)

    def send_phone_number_confirmation_sms(self, generate_new=False):
        if generate_new or self.phone_number_confirmation_pin == 0:
            self.phone_number_confirmation_pin = random.randint(0, 999999)
        self.last_phone_number_confirmation_pin_sent = timezone.now()
        self.save()
        with translation.override(self.preferred_language):
            moses_settings.SEND_SMS_HANDLER(
                self.phone_number,
                _("Phone number confirmation PIN: ") + str(self.phone_number_confirmation_pin).zfill(6)
            )

    def send_phone_number_candidate_confirmation_sms(self, generate_new=False):
        if not self.phone_number_candidate:
            return
        if generate_new or self.phone_number_candidate_confirmation_pin == 0:
            self.phone_number_candidate_confirmation_pin = random.randint(0, 999999)
        if (
                self.last_phone_number_candidate_confirmation_pin_sent is not None and
                self.last_phone_number_candidate_confirmation_pin_sent > timezone.now() - timedelta(
            minutes=moses_settings.MINUTES_BETWEEN_CONFIRMATION_PIN_SMS
        )):
            return
        self.last_phone_number_candidate_confirmation_pin_sent = timezone.now()
        self.save()
        with translation.override(self.preferred_language):
            moses_settings.SEND_SMS_HANDLER(
                self.phone_number_candidate,
                _("Phone number confirmation PIN: ") + str(
                    self.phone_number_candidate_confirmation_pin).zfill(
                    6))

    def try_to_confirm_credential(self, credential: Credential, main_pin_str: str, candidate_pin_str: str):
        match credential:
            case Credential.PHONE_NUMBER:
                attempts_field = 'phone_number_confirmation_attempts'
                current_credential_field = 'phone_number'
                candidate_credential_field = 'phone_number_candidate'
                current_credential_confirmation_field = 'is_phone_number_confirmed'
                max_attempts_limit = moses_settings.MAX_PHONE_NUMBER_CONFIRMATION_ATTEMPTS
                current_pin_field = 'phone_number_confirmation_pin'
                candidate_pin_field = 'phone_number_candidate_confirmation_pin'
            case Credential.EMAIL:
                attempts_field = 'email_confirmation_attempts'
                current_credential_field = 'email'
                candidate_credential_field = 'email_candidate'
                current_credential_confirmation_field = 'is_email_confirmed'
                max_attempts_limit = moses_settings.MAX_EMAIL_CONFIRMATION_ATTEMPTS
                current_pin_field = 'email_confirmation_pin'
                candidate_pin_field = 'email_candidate_confirmation_pin'
            case _:
                raise ValueError('invalid_credential')
        if getattr(self, attempts_field) == max_attempts_limit:
            return False, False
        received_pin, received_candidate_pin = int(main_pin_str or '0'), int(candidate_pin_str or '0')
        is_main_pin_correct = received_pin == getattr(self, current_pin_field)
        is_candidate_pin_correct = None
        if getattr(self, candidate_credential_field):
            is_candidate_pin_correct = getattr(self, candidate_pin_field) == received_candidate_pin
        if is_main_pin_correct and (is_candidate_pin_correct is None or is_candidate_pin_correct):
            setattr(self, current_pin_field, 0)
            setattr(self, candidate_pin_field, 0)
            setattr(self, current_credential_confirmation_field, True)
            setattr(self, attempts_field, 0)
            if candidate := getattr(self, candidate_credential_field):
                setattr(self, current_credential_field, candidate)
                setattr(self, candidate_credential_field, '')
            self.save()
        else:
            setattr(self, attempts_field, getattr(self, attempts_field) + 1)
            self.save()
        return is_main_pin_correct, is_candidate_pin_correct

    def send_email_confirmation_email(self, generate_new=False):
        if generate_new or self.email_confirmation_pin == 0:
            self.email_confirmation_pin = random.randint(0, 999999)
            self.save()
        with translation.override(self.preferred_language):
            send_mail(_("Email confirmation PIN"),
                      _("Your email confirmation PIN is: ") + str(self.email_confirmation_pin),
                      'noreply@' + moses_settings.DOMAIN, [self.email])

    def send_email_candidate_confirmation_email(self, generate_new=False):
        if not self.email_candidate:
            return
        if generate_new or self.email_candidate_confirmation_pin == 0:
            self.email_candidate_confirmation_pin = random.randint(0, 999999)
            self.save()
        with translation.override(self.preferred_language):
            send_mail(_("Email confirmation PIN"),
                      _("Your email confirmation PIN is: ") + str(self.email_candidate_confirmation_pin),
                      'noreply@' + django_settings.DOMAIN, [self.email_candidate])

    def __str__(self):
        return f'{self.first_name} {self.last_name}'



SITE_WITH_DOMAIN_DOES_NOT_EXIST = 'site_with_domain_does_not_exist'
INCORRECT_PHONE_NUMBER = 'incorrect_phone_number'
INCORRECT_CONFIRMATION_PIN = 'incorrect_confirmation_pin'
EMAIL_ALREADY_REGISTERED_ON_DOMAIN = 'email_already_registered_on_domain'
PHONE_NUMBER_ALREADY_REGISTERED_ON_DOMAIN = 'phone_number_already_registered_on_domain'
INVALID_PASSWORD = 'invalid_password'
CREDENTIAL_NOT_CONFIRMED = 'credential_not_confirmed'
USER_WITH_PROVIDED_CREDENTIALS_DOES_NOT_REGISTERED_ON_SPECIFIED_DOMAIN = 'user_with_provided_credentials_does_not_registered_on_specified_domain'
TOO_FREQUENT_SMS_REQUESTS = 'too_frequent_sms_requests'



from django.contrib.auth import authenticate
from django.contrib.auth.models import Group
from django.contrib.auth.password_validation import validate_password
from django.contrib.sites.models import Site
from django.core import exceptions as django_exceptions
from django.core.validators import EmailValidator
from django.db import IntegrityError, transaction
from django.utils.translation import gettext as _
from djoser import constants
from rest_framework import serializers, exceptions
from rest_framework.fields import CharField
from rest_framework.serializers import raise_errors_on_nested_writes, ModelSerializer, Serializer
from rest_framework.utils import model_meta
from rest_framework_simplejwt.serializers import TokenObtainSerializer
from rest_framework_simplejwt.tokens import RefreshToken

from moses import errors
from moses.conf import settings as moses_settings
from moses.models import CustomUser


class PinSerializer(Serializer):
    pin = CharField()
    candidate_pin = CharField(required=False)


class MFASerializer(Serializer):
    otp = CharField()
    secret_key = CharField()
    action = CharField(required=False)


class GroupSerializer(ModelSerializer):
    class Meta:
        model = Group
        fields = '__all__'


class PasswordResetSerializer(serializers.Serializer):
    email = serializers.EmailField(required=False)
    phone_number = serializers.CharField(required=False)

    default_error_messages = {
        'email_not_found': constants.Messages.EMAIL_NOT_FOUND,
        'phone_number_not_found': _('User with given phone number does not exist.')
    }

    def validate_email(self, value):
        users = self.context['view'].get_users_by_email(value)
        if not users.exists():
            self.fail('email_not_found')
        else:
            return value

    def validate_phone_number(self, value):
        users = self.context['view'].get_users_by_phone_number(value)
        if not users.exists():
            self.fail('phone_number_not_found')
        else:
            return value


class ShortCustomUserSerializer(serializers.ModelSerializer):
    income_subscription = serializers.BooleanField(read_only=True)
    outcome_subscription = serializers.BooleanField(read_only=True)

    class Meta:
        model = CustomUser
        fields = ['id', 'first_name', 'last_name', 'userpic', 'outcome_subscription', 'income_subscription']
        read_only_fields = ['id', 'first_name', 'last_name', 'userpic', 'income_subscription',
                            'outcome_subscription']


class PublicCustomUserSerializer(serializers.ModelSerializer):
    class Meta:
        model = CustomUser
        fields = [
            'id',
            'first_name',
            'last_name',
        ]


class PrivateCustomUserSerializer(serializers.ModelSerializer):
    is_mfa_enabled = serializers.SerializerMethodField()

    def get_is_mfa_enabled(self, obj):
        return len(obj.mfa_secret_key) > 0


    def update(self, instance, validated_data):
        raise_errors_on_nested_writes('update', self, validated_data)
        info = model_meta.get_field_info(instance)
        if 'phone_number' in validated_data:
            _phone_number = validated_data.pop('phone_number')
            if _phone_number == instance.phone_number:
                instance.phone_number_candidate = ''
                instance.phone_number_confirm_pin = 0
                instance.phone_number_candidate_confirm_pin = 0
                instance.phone_number_confirm_attempts = 0
                instance.save()
            elif _phone_number != (instance.phone_number_candidate or instance.phone_number):
                if instance.is_phone_number_confirmed:
                    instance.phone_number_candidate = _phone_number
                    instance.phone_number_confirm_attempts = 0
                    instance.save()
                    instance.send_phone_number_confirmation_sms(generate_new=True)
                    instance.send_phone_number_candidate_confirmation_sms(generate_new=True)
                else:
                    instance.phone_number = _phone_number
                    instance.phone_number_confirm_attempts = 0
                    instance.save()
                    instance.send_phone_number_confirmation_sms(generate_new=True)
        if 'email' in validated_data:
            _email = validated_data.pop('email')
            if _email == instance.email:
                instance.email_candidate = ''
                instance.email_confirm_pin = 0
                instance.email_candidate_confirm_pin = 0
                instance.email_confirm_attempts = 0
                instance.save()
            elif _email != (instance.email_candidate or instance.email):
                if instance.is_email_confirmed:
                    instance.email_candidate = _email
                    instance.email_confirm_attempts = 0
                    instance.save()
                    instance.send_email_confirmation_email(generate_new=True)
                    instance.send_email_candidate_confirmation_email(generate_new=True)
                else:
                    instance.email = _email
                    instance.email_confirm_attempts = 0
                    instance.save()
                    instance.send_email_confirmation_email(generate_new=True)
        for attr, value in validated_data.items():
            if attr in info.relations and info.relations[attr].to_many:
                field = getattr(instance, attr)
                field.set(value)
            else:
                setattr(instance, attr, value)
        instance.save()
        return instance

    class Meta:
        model = CustomUser
        fields = [
            'id',
            'userpic',
            'email',
            'email_candidate',
            'first_name',
            'last_name',
            'phone_number',
            'phone_number_candidate',
            'is_phone_number_confirmed',
            'is_email_confirmed',
            'preferred_language',
            'is_mfa_enabled'
        ]


def site_with_domain_exists(value):
    if not Site.objects.filter(domain=value).exists():
        raise serializers.ValidationError(errors.SITE_WITH_DOMAIN_DOES_NOT_EXIST)

def validate_phone_number_with_provided_validator(value):
    if not Site.objects.filter(domain=value).exists():
        raise serializers.ValidationError(errors.SITE_WITH_DOMAIN_DOES_NOT_EXIST)


class CustomUserCreateSerializer(serializers.ModelSerializer):
    phone_number = serializers.CharField(validators=[moses_settings.PHONE_NUMBER_VALIDATOR])
    email = serializers.CharField(validators=[EmailValidator])
    domain = serializers.CharField(validators=[site_with_domain_exists], write_only=True)
    password = serializers.CharField(
        style={'input_type': 'password'},
        write_only=True,
        validators=[validate_password]
    )
    default_error_messages = {
        'cannot_create_user': constants.Messages.CANNOT_CREATE_USER_ERROR,
    }

    class Meta:
        model = CustomUser
        fields = [
            'email',
            'first_name',
            'last_name',
            'phone_number',
            'password',
            'preferred_language',
            'domain'
        ]

    def validate(self, attrs):
        if 'email' not in attrs or CustomUser.objects.filter(
                site__domain=attrs['domain'],
                email=attrs['email']
        ).exists():
            raise serializers.ValidationError(errors.EMAIL_ALREADY_REGISTERED_ON_DOMAIN)
        if 'phone_number' not in attrs or CustomUser.objects.filter(
                site__domain=attrs['domain'],
                phone_number=attrs['phone_number']
        ).exists():
            raise serializers.ValidationError(errors.PHONE_NUMBER_ALREADY_REGISTERED_ON_DOMAIN)
        attrs['site'] = Site.objects.get(domain=attrs.pop('domain'))
        user_attrs = {k: v for k, v in attrs.items() if k != 'inviter_id'}
        user = CustomUser(**user_attrs)
        return attrs

    def create(self, validated_data):
        try:
            user = self.perform_create(validated_data)
        except IntegrityError as e:
            self.fail('cannot_create_user')
        return user

    def perform_create(self, validated_data):
        with transaction.atomic():
            user = CustomUser.objects.create_user(**validated_data)
            user.preferred_language = 'en'
        return user


class TokenObtainPairSerializer(TokenObtainSerializer):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['otp'] = serializers.CharField(required=False, allow_blank=True, allow_null=True)
        self.fields['domain'] = serializers.CharField(required=False, allow_blank=True, allow_null=True)

    @classmethod
    def get_token(cls, user):
        return RefreshToken.for_user(user)

    def validate(self, attrs):
        authenticate_kwargs = {
            self.username_field: attrs[self.username_field],
            'password': attrs['password'],
            'otp': attrs.get('otp'),
            'domain': attrs.get('domain')
        }
        try:
            authenticate_kwargs['request'] = self.context['request']
        except KeyError:
            pass

        self.user = authenticate(**authenticate_kwargs)

        if self.user is None or not self.user.is_active:
            raise exceptions.AuthenticationFailed(
                self.error_messages['no_active_account'],
                'no_active_account',
            )

        data = {}

        refresh = self.get_token(self.user)

        data['refresh'] = str(refresh)
        data['access'] = str(refresh.access_token)
        return data



# Generated by Django 4.2.7 on 2023-11-07 07:27

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import moses.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomUser',
            fields=[
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('email', models.EmailField(max_length=254)),
                ('email_candidate', models.EmailField(blank=True, max_length=254, verbose_name='Email candidate')),
                ('is_email_confirmed', models.BooleanField(default=False, verbose_name='Is email confirmed')),
                ('email_confirmation_pin', models.PositiveIntegerField(default=0, verbose_name='Email confirm PIN')),
                ('email_candidate_confirmation_pin', models.PositiveIntegerField(default=0, verbose_name='Email candidate confirm PIN')),
                ('email_confirmation_attempts', models.PositiveSmallIntegerField(default=0, verbose_name='Email confirm attempts')),
                ('phone_number', models.CharField(max_length=20, verbose_name='Phone number')),
                ('phone_number_candidate', models.CharField(blank=True, max_length=20, verbose_name='Phone number candidate')),
                ('is_phone_number_confirmed', models.BooleanField(default=False, verbose_name='Is phone number confirmed')),
                ('phone_number_confirmation_pin', models.PositiveIntegerField(default=0, verbose_name='Phone number confirm PIN')),
                ('phone_number_candidate_confirmation_pin', models.PositiveIntegerField(default=0, verbose_name='Phone number candidate confirm PIN')),
                ('phone_number_confirmation_attempts', models.PositiveSmallIntegerField(default=0, verbose_name='Phone number confirm attempts')),
                ('last_password_reset_sms_sent_at', models.DateTimeField(blank=True, null=True, verbose_name='Last password reset sms sent at')),
                ('first_name', models.CharField(blank=True, max_length=200, verbose_name='First name')),
                ('last_name', models.CharField(blank=True, max_length=200, verbose_name='Last name')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is active')),
                ('is_staff', models.BooleanField(default=False, verbose_name='Is staff')),
                ('preferred_language', models.CharField(choices=[('en', 'English')], default='en', max_length=10, verbose_name='Preferred language')),
                ('created_at', models.DateTimeField(blank=True, default=django.utils.timezone.now, null=True, verbose_name='Created at')),
                ('mfa_secret_key', models.CharField(blank=True, default='', max_length=160)),
                ('last_phone_number_confirmation_pin_sent', models.DateTimeField(blank=True, null=True)),
                ('last_phone_number_candidate_confirmation_pin_sent', models.DateTimeField(blank=True, null=True)),
                ('userpic', models.ImageField(blank=True, null=True, upload_to='images/userpics/')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='users', to='sites.site')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'User',
                'verbose_name_plural': 'Users',
            },
            managers=[
                ('objects', moses.models.CustomUserManager()),
            ],
        ),
        migrations.AddConstraint(
            model_name='customuser',
            constraint=models.UniqueConstraint(fields=('site', 'phone_number'), name='one_phone_number_per_site'),
        ),
        migrations.AddConstraint(
            model_name='customuser',
            constraint=models.UniqueConstraint(fields=('site', 'email'), name='one_email_per_site'),
        ),
    ]



from django.conf import settings as django_settings
from django.test.signals import setting_changed
from django.utils.functional import LazyObject
from django.utils.module_loading import import_string
from django.utils.translation import gettext as _

MOSES_SETTINGS_NAMESPACE = "MOSES"


class ObjDict(dict):
    def __getattribute__(self, item):
        try:
            val = self[item]
            if isinstance(val, str):
                val = import_string(val)
            elif isinstance(val, (list, tuple)):
                val = [import_string(v) if isinstance(v, str) else v for v in val]
            self[item] = val
        except KeyError:
            val = super(ObjDict, self).__getattribute__(item)

        return val


default_settings = {
    "PHONE_NUMBER_VALIDATOR": None,
    "DEFAULT_LANGUAGE": None,
    "SEND_SMS_HANDLER": None,
    "DOMAIN": None,
    "URL_PREFIX": None,
    "IP_HEADER": None,
    "MINUTES_BETWEEN_CONFIRMATION_PIN_SMS": 1,
    "MAX_PHONE_NUMBER_CONFIRMATION_ATTEMPTS": 3,
    "MAX_EMAIL_CONFIRMATION_ATTEMPTS": 3,
    "LANGUAGE_CHOICES": (
        (1, _("English")),
    ),
}

SETTINGS_TO_IMPORT = ["SEND_SMS_HANDLER", "PHONE_NUMBER_VALIDATOR"]


class Settings:
    def __init__(self, default_settings, explicit_overriden_settings: dict = None):
        if explicit_overriden_settings is None:
            explicit_overriden_settings = {}

        overriden_settings = (
                getattr(django_settings, MOSES_SETTINGS_NAMESPACE, {})
                or explicit_overriden_settings
        )

        self._load_default_settings()
        self._override_settings(overriden_settings)
        self._init_settings_to_import()

    def _load_default_settings(self):
        for setting_name, setting_value in default_settings.items():
            if setting_name.isupper():
                setattr(self, setting_name, setting_value)

    def _override_settings(self, overriden_settings: dict):
        for setting_name, setting_value in overriden_settings.items():
            value = setting_value
            if isinstance(setting_value, dict):
                value = getattr(self, setting_name, {})
                value.update(ObjDict(setting_value))
            setattr(self, setting_name, value)

    def _init_settings_to_import(self):
        for setting_name in SETTINGS_TO_IMPORT:
            value = getattr(self, setting_name)
            if isinstance(value, str):
                setattr(self, setting_name, import_string(value))


class LazySettings(LazyObject):
    def _setup(self, explicit_overriden_settings=None):
        self._wrapped = Settings(default_settings, explicit_overriden_settings)


settings = LazySettings()


def reload_moses_settings(*args, **kwargs):
    global settings
    setting, value = kwargs["setting"], kwargs["value"]
    if setting == MOSES_SETTINGS_NAMESPACE:
        settings._setup(explicit_overriden_settings=value)


setting_changed.connect(reload_moses_settings)
